/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*-  */
/* is_maildir.c
 *
 * Copyright (C) 2019  Giam Teck Choon <choon@choon.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 */

#include <errno.h>
#include <bglibs/str.h>
#include "is_dir.h"

/*
 * Return 1 if found otherwise return 0.
 * Return -1 if error and errno will be set. 
 */
int is_maildir(const char *path)
{
	if (is_dir(path)) {
		static str t = {0,0,0};
		if (!str_copy2s(&t, path, "/tmp")) {
			errno = ENOMEM;
			str_free(&t);
			return -1;
		}
		if (!is_dir(t.s)) {
			str_free(&t);
			return 0;
		}
		if (!str_copy2s(&t, path, "/new")) {
			errno = ENOMEM;
			str_free(&t);
			return -1;
		}
		if (!is_dir(t.s)) {
			str_free(&t);
			return 0;
		}
		if (!str_copy2s(&t, path, "/cur")) {
			errno = ENOMEM;
			str_free(&t);
			return -1;
		}
		if (!is_dir(t.s)) {
			str_free(&t);
			return 0;
		}
		str_free(&t);
		return 1;
	}
	return 0;
}
