/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*-  */
/* mktmpdir.c
 *
 * Copyright (C) 2019  Giam Teck Choon <choon@choon.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 */

#include <sys/stat.h>
#include <sys/types.h>
#include <string.h>
#include <errno.h>
#include <bglibs/str.h>
#include "is_dir.h"
#include "str_random.h"

/*
 * On success, zero is returned.  On error, -1 is returned, and errno is
 * set appropriately.
 */
int mktmpdir(const char *path, str *s)
{
	if (!is_dir(path)) {
		errno = ENOENT;
		return -1;
	}
	static str t = {0,0,0};
	if (str_random(&t, 16) != 0) {
		str_free(&t);
		return -1;
	}
	static str p = {0,0,0};
	if (!str_copy3s(&p, path, "/", t.s)) {
		str_free(&t);
		str_free(&p);
		errno = ENOMEM;
		return -1;
	}
	if (mkdir(p.s, 0700) != 0) {
		str_free(&t);
		str_free(&p);
		return -1;
	}
	if (s->s) str_truncate(s, 0);
	if (!str_copyb(s, p.s, p.len)) {
		str_free(&t);
		str_free(&p);
		errno = ENOMEM;
		return -1;
	}
	str_free(&t);
	str_free(&p);
	return 0;
}
