/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*-  */
/* rmrf.c
 *
 * Copyright (C) 2019  Giam Teck Choon <choon@choon.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 */

#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include <errno.h>
#include <string.h>
#include <bglibs/str.h>
#include "is_dir.h"

/*
 * On success, zero is returned.  On error, -1 is returned, and errno is
 * set appropriately.
 */
int rmrf(const char *path)
{
	DIR *dir;
	direntry *entry;
	/*
	 * On error, NULL is returned, and errno is set appropriately.
	 */
	if ((dir = opendir(path)) == NULL) { /* <sys/types.h> and <dirent.h> */
		return -1;
	}
	str fullname = {0,0,0};
	while ((entry = readdir(dir)) != NULL) {
		const char* name = entry->d_name;
		if (name[0] == '.' && (name[1] == 0
				       || (name[1] == '.'
					   && name[2] == 0))) {
			continue;
		}
		if (!str_copy3s(&fullname, path, "/", name)) {
			str_free(&fullname);
			errno = ENOMEM;
			return -1;
		}
		if (is_dir(fullname.s)) {
			if (rmrf(fullname.s) != 0) {
				str_free(&fullname);
				return -1;
			}
			continue;
		}
		if (unlink(fullname.s) != 0) {
			str_free(&fullname);
			return -1;
		}
	}
	closedir(dir);
	str_free(&fullname);
	if (rmdir(path) != 0) {
		return -1;
	}
	return 0;
}
