/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*-  */
/* str_random.c
 *
 * Copyright (C) 2019  Giam Teck Choon <choon@choon.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 */

#include <stdlib.h>
#include <string.h>
#include <sys/time.h>
#include <errno.h>
#include <bglibs/str.h>

/*
 * Generate random string of the given length.
 *
 * :param str: struct str to record the random string.
 * :param len: The length of the resulted random string.
 * :return: 0 on success, or -1 on error, with errno set to indicate the
 *          cause of the error.
 */
int str_random(str *s, size_t len)
{
	struct timeval t1;
	int r;
	r = gettimeofday(&t1, NULL); /* <sys/time.h> */
	if (r != 0) {
		return r;
	}
	srand(t1.tv_usec * t1.tv_sec + len); /* <stdlib.h> */
	static const char charset[] = "0123456789"
				      "abcdefghijklmnopqrstuvwxyz"
				      "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
	unsigned int nchars = strlen(charset); /* <string.h> */
	if (s->s) {
		if (!str_truncate(s, 0)) {
			errno = ENOMEM;
			return -1;
		}
	}
	size_t i;
	for (i=0; i<len; i++) {
		if (!str_catc(s, charset[(unsigned int) ((rand() /
			((double) RAND_MAX + 1)) * nchars)])) {
			errno = ENOMEM;
			return -1;
		}
	}
	if (s->len == len) {
		return 0;
	}
	errno = ENODATA;
	return -1;
}
